use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007000
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
JOBSUFFIX
jobname
messager
REFSPEC
shortlog
Chaves
CheckCommitAuthor
Gerrit
GitWeb
Gravatar
Leite
Mendonça
VCS
de
githooks
patchset
userenv
Readonly
committer
committers
mailmap
AnnoCPAN
CPAN
io
IO
Github
HTTPS
JSON
POSIX
Params
Storable
Subdirectory
TODO
Timestamp
alnum
ascii
aws
cas
dir
filename
http
https
json
licensable
lv
msg
multi
params
runtime
scalability
semafores
startup
subitem
subparam
succcessful
successfull
timestamp
tradename
unreserves
url
webhooks
wildcard
wildcards
Mikko
Koivunalho
mikko
lib
Git
MoreHooks
CheckCommitAuthorFromMailmap
CheckCommitBase
CheckIndent
TriggerJenkins
